/*
 * Copyright (c) 1995-2001 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#ifndef _SYS_DMFE_IMPL_H
#define	_SYS_DMFE_IMPL_H

#pragma ident	"@(#)dmfe_impl.h	1.1	01/08/17 SMI"

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <sys/stream.h>
#include <sys/strsun.h>
#include <sys/stat.h>
#include <sys/pci.h>
#include <sys/note.h>
#include <sys/modctl.h>
#include <sys/kstat.h>
#include <sys/ethernet.h>
#include <sys/dlpi.h>
#include <sys/devops.h>
#include <sys/debug.h>
#include <sys/conf.h>

#include <inet/common.h>
#include <inet/nd.h>
#include <inet/mi.h>

#include <sys/dditypes.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include <sys/gld.h>

#include <sys/mii.h>
#include <sys/dmfe.h>

/*
 * Describes the state of a descriptor ring
 *
 * NOTE: n_free and next_busy are only used for the Tx descriptors
 * and are not valid on the receive side.
 */
typedef struct desc_state {
	uint32_t	n_desc;		/* total number of descriptors	*/
	uint32_t	n_free;		/* number of free descriptors	*/
	uint32_t	next_free;	/* next index to use or check	*/
	uint32_t	next_busy;	/* next index to reclaim	*/
} desc_state_t;

/*
 * Describes one chunk of allocated DMA-able memory
 */
typedef struct dma_area {
	ddi_dma_handle_t	dma_hdl;
	ddi_acc_handle_t	acc_hdl;
	size_t			alength;	/* allocated size	*/
	caddr_t			mem_va;		/* CPU VA of memory	*/
	uint32_t		spare1;
	uint32_t		mem_dvma;	/* DVMA addr of memory	*/
	caddr_t			setup_va;
	uint32_t		spare2;
	uint32_t		setup_dvma;
	int			spare3;
	int			ncookies;
} dma_area_t;

/*
 * Per-instance soft-state structure
 */
struct dmfeinstance {
	dev_info_t		devinfo;
	ddi_acc_handle_t	io_handle;	/* ddi I/O handle	*/
	caddr_t			io_reg;		/* mapped registers	*/
	uint32_t		opmode;		/* operating mode shadow */
	uint32_t		imask;		/* interrupt mask shadow */

	desc_state_t		tx;		/* describes Tx ring	*/
	desc_state_t		rx;		/* describes Rx ring	*/

	dma_area_t		tx_desc;	/* transmit descriptors	*/
	dma_area_t		tx_buff;	/* transmit buffers	*/
	dma_area_t		rx_desc;	/* receive descriptors	*/
	dma_area_t		rx_buff;	/* receive buffers	*/

	int			chip_type;	/* chip ID		*/
	int			revision;	/* chip revision	*/
	int			debug;		/* per-instance debug	*/
	int			promisc;	/* in promiscuous mode	*/
	uint32_t		chip_state;

	mii_handle_t		mii;
	int			mii_speed;
	int			mii_duplex;
	int			mii_up;
	int			phyaddr;
	int			leaf;
	media_block_t		*selected_media_block;
	SROM_FORMAT		sr;

	caddr_t			g_nd;
	uint_t			param_display[DMFE_PARAM_CNT];
	dmfeparam_t		param_arr[DMFE_PARAM_CNT];

	uint8_t			multicast_cnt[MCASTBUF_SIZE];
};

/*
 * Chip state bits ...
 */
#define	CHIP_STATE_READY	0
#define	CHIP_STATE_RESET	1
#define	CHIP_STATE_PHYRESET	2
#define	CHIP_STATE_LINKCHG	4
#define	CHIP_STATE_RUNT_PENDING	8

/*
 * Options to dmfe_enable_interrupts()
 */
#define	ENINT_DEFAULT		0
#define	ENINT_TX_DONE		1
#define	ENINT_RUNT_TIMER	2
#define	ENINT_REPROGRAM		3

/*
 * Default values for number of Tx/Rx descriptors
 */
#define	DMFE_TX_DESC		32	/* Should be a multiple of 4 <= 256 */
#define	DMFE_RX_DESC		32	/* Should be a multiple of 4 <= 256 */

/*
 * Copy an ethernet address
 */
#define	ethaddr_copy(src, dst)	bcopy((src), (dst), ETHERADDRL)

/*
 * The macros below that the current instance structure is pointed
 * to by a variable named 'dmfep' - make sure you have such a thing
 * in scope (and initialised) when using any of these!
 */

#define	NEXT_TX_INDEX(index) ((index)+1 < dmfep->tx.n_desc ? (index)+1 : 0)
#define	NEXT_RX_INDEX(index) ((index)+1 < dmfep->rx.n_desc ? (index)+1 : 0)

#define	param_linkup		(dmfep->param_arr[0].param_val)
#define	param_speed		(dmfep->param_arr[1].param_val)
#define	param_mode		(dmfep->param_arr[2].param_val)
#define	param_autoneg		(dmfep->param_arr[3].param_val)
#define	param_anar_100fdx	(dmfep->param_arr[4].param_val)
#define	param_anar_100hdx	(dmfep->param_arr[5].param_val)
#define	param_anar_10fdx	(dmfep->param_arr[6].param_val)
#define	param_anar_10hdx	(dmfep->param_arr[7].param_val)

/*
 * Debugging ...
 */
#ifdef	DEBUG
#define	DMFEDEBUG		1
#else
#define	DMFEDEBUG		0
#endif

/* debug flags */
#define	DMFETRACE		0x0001
#define	DMFEERRS		0x0002
#define	DMFERECV		0x0004
#define	DMFEDDI			0x0008
#define	DMFESEND		0x0010
#define	DMFEINT			0x0020
#define	DMFEOPN			0x0040
#define	DMFESENSE		0x0080
#define	DMFESTAT		0x0100
#define	DMFEXMIT		0x0200
#define	DMFENDD			0x0400
#define	DMFEMII			0x0800
#define	DMFELOOP		0x1000
#define	DMFEPROMISC		0x2000
#define	DMFESROM		0x4000
#define	DMFEINIT		0x8000

#if	DMFEDEBUG

extern uint32_t dmfe_debug;

/*
 * Define DMFE_DBG to be the relevant flag from the set above before
 * using the DMFE_DEBUG() or DMFE_GDEBUG() macros.  The 'G' versions
 * look at the Global debug flag word (dmfe_debug); the non-G versions
 * look in the per-instance data (dmfep->debug).
 */

#define	DMFE_DEBUG(args)	do {					\
					if (dmfep->debug & (DMFE_DBG))	\
						cmn_err args;		\
					_NOTE(CONSTANTCONDITION)	\
				} while (0)

#define	DMFE_TRACE(args)	do {					\
					if (dmfep->debug & (DMFETRACE))	\
						cmn_err args;		\
					_NOTE(CONSTANTCONDITION)	\
				} while (0)

#define	DMFE_GDEBUG(args)	do {					\
					if (dmfe_debug & (DMFE_DBG))	\
						cmn_err args;		\
					_NOTE(CONSTANTCONDITION)	\
				} while (0)

#define	DMFE_GTRACE(args)	do {					\
					if (dmfe_debug & (DMFETRACE))	\
						cmn_err args;		\
					_NOTE(CONSTANTCONDITION)	\
				} while (0)


#else

#define	DMFE_DEBUG(args)	do ; _NOTE(CONSTANTCONDITION) while (0)
#define	DMFE_TRACE(args)	do ; _NOTE(CONSTANTCONDITION) while (0)
#define	DMFE_GDEBUG(args)	do ; _NOTE(CONSTANTCONDITION) while (0)
#define	DMFE_GTRACE(args)	do ; _NOTE(CONSTANTCONDITION) while (0)

#endif	/* DMFEDEBUG */


/*
 * Inter-source-file linkage ...
 */

/* dmfe_main.c */
uint32_t dmfe_chip_get32(struct dmfeinstance *dmfep, off_t offset);
void dmfe_chip_put32(struct dmfeinstance *dmfep, off_t offset, uint32_t value);
void dmfe_restart(gld_mac_info_t *macinfo, struct dmfeinstance *dmfep);

/* dmfe_mii.c */
int dmfe_find_phy(gld_mac_info_t *macinfo, struct dmfeinstance *dmfep);
void dmfe_start_portmon(gld_mac_info_t *macinfo);
void dmfe_phy_cleanup(struct dmfeinstance *dmfep);

/* dmfe_ndd.c */
int dmfe_init_xfer_params(struct dmfeinstance *dmfep);
void dmfe_process_ndd_ioctl(queue_t *wq, mblk_t *mp, int cmd);
void dmfe_param_cleanup(struct dmfeinstance *dmfep);

/* dmfe_srom.c */
void dmfe_setup_legacy_blocks(void);
int dmfe_init_srom(struct dmfeinstance *dmfep, dev_info_t *dip);

/* various */
extern LEAF_FORMAT dmfe_leaf_phy_legacy;
extern char dmfe_ident[];
/* kernel */
extern int
ddi_create_internal_pathname(dev_info_t *dip, char *name, int spec_type,
	minor_t minor_num);

#ifdef __cplusplus
}
#endif

#endif	/* _SYS_DMFE_IMPL_H */
